// Copyright 2019 Patric Ljung
// Linkoping University, Media and Information Technology, Visualization Center C
// License: BSD-3-Clause (https://opensource.org/licenses/BSD-3-Clause)

#pragma once

#include "CoreMinimal.h"
#include "ProceduralMeshComponent.h"
// TODO: Move to .cpp file and provide forward declaration here instead
#include "Materials/MaterialInterface.h"

#include "DomeDisplayComponent.generated.h"

/**
 * 
 */
UCLASS(BlueprintType, Category = "Dome",
       Meta = (BlueprintSpawnableComponent, Tooltip = "A component that dynamically creates a UV-mapped dome hemisphere mesh"))
class DOMEVIEWERVR_API UDomeDisplayComponent : public USceneComponent
{
	GENERATED_BODY()
	
public:
    UDomeDisplayComponent();
    ~UDomeDisplayComponent();

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Dome",
              Meta = (Tooltip = "The radius of the dome"))
    float Radius;
    
    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Dome",
              Meta = (Tooltip = "How many degrees is the dome tilted forward"))
    float TiltAngle;

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Dome",
              Meta = (Tooltip = "How many degrees the hemisphere spans, 180 degrees makes it a full hemishphere"))
    float HemisphereDegrees;

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Dome",
              Meta = (Tooltip = "Mesh resolution, degrees per bottom edge"))
    float MeshEdgeDegrees;

    UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Dome",
              Meta = (Tooltip = "Dome display mesh material"))
    UMaterialInterface * Material;

//    UPROPERTY(EditAnywhere, BlueprintRead, Category = "Dome",
//              Meta = (Tooltip = "Dome display mesh material"))
    UPROPERTY()
    UProceduralMeshComponent * DomeMesh;

public:
    UFUNCTION(BlueprintCallable, Category = "Dome",
              Meta = (Tooltip = "Update the dome mesh using current settings"))
    bool UpdateDomeMesh();
    
protected:
    // TODO: Add method to capture editor changes and call BuildDomeMesh

    void BuildDomeMesh();

    TArray<FVector>     MeshVerts;
    TArray<int32>       MeshTris;
    TArray<FVector>     MeshNormals;
    TArray<FVector2D>   MeshUVs;
    TArray<FColor>      MeshColors;
    TArray<FProcMeshTangent> MeshTangents;

};
